<?php

namespace Drupal\test_users\Commands;

use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\user\Entity\User;
use Drupal\user\RoleInterface;
use Drush\Commands\DrushCommands;
use \Exception;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class TestUsersCommands extends DrushCommands {

  /**
   * Create test users.
   *
   * Creates a user for each role in the system (or adds the role to an existing
   * user named after the role).
   *
   * @usage drush test-users:create
   *   Create test users.
   * @validate-module-enabled user
   *
   * @command test-users:create
   * @aliases ctu,createtestusers
   */
  public function createTestUsers() {
    $roles = user_roles(TRUE);

    foreach ($roles as $rid => $role) {
      $name = $role->label();
      
      // If the role has no label, set the name to the role ID, which is
      // also somewhat descriptive.
      if (empty($name)) {
        $name = $rid;
      }

      $mail = Html::getClass($name) . '@example.com';

      /** @var \Drupal\user\UserInterface $account */
      $account = user_load_by_name($name);
      if ($account) {
        if ($account->getEmail() !== $mail) {
          if ($rid !== RoleInterface::AUTHENTICATED_ID) {
            $account->addRole($rid);
          }
          $message = 'Added role to existing user: ' . $name;
        }
        else {
          $message = 'Updated existing user for role: ' . $name;
        }
      }
      else {
        $account = User::create([
          'name' => $name,
          'pass' => 'password',
          'mail' => $mail,
          'status' => 1,
        ]);
        if ($rid !== RoleInterface::AUTHENTICATED_ID) {
          $account->addRole($rid);
        }
        $account->enforceIsNew();

        $message = 'Created new user for role: ' . $name;
      }
      try {
        $account->save();
        $this->logger()->success($message);
      }
      catch (Exception $e) {
        $this->logger()->error('Failed to create user for role: ' . $name);
        throw $e;
      }
    }
  }

  /**
   * Delete test users.
   *
   * Delete users as created by test-users:create. Will ask for confirmation
   * after determining which users would be deleted. Take note that it finds
   * users exclusively based on their name matching the name of a role.
   *
   * @usage drush test-users:delete
   *   Delete test users.
   * @validate-module-enabled user
   *
   * @command test-users:delete
   * @aliases dtu,deletetestusers
   */
  public function deleteTestUsers() {
    $roles = user_roles(TRUE);

    $accounts = [];
    foreach ($roles as $role) {
      $name = $role->label();
      /** @var \Drupal\user\UserInterface $account */
      $account = user_load_by_name($name);
      if ($account && $account->id() > 1) {
        $accounts[$name] = $account;
      }
    }

    if (empty($accounts)) {
      $this->logger()->notice('No test users found.');
    }
    else {
      $userList = implode(PHP_EOL, array_keys($accounts));
      $msg = 'This will delete these users:' . PHP_EOL . $userList . PHP_EOL . 'Are you sure?';
      $sure = $this->io()->confirm($msg);
      if ($sure) {
        foreach ($accounts as $name => $account) {
          try {
            $account->delete();
            $this->logger()->notice('Deleted user for role: ' . $name);
          }
          catch (EntityStorageException $e) {
            $this->logger()->error('Failed to delete user for role ' . $name . '. ' . $e->getMessage());
          }
        }
      }
    }
  }

  /**
   * Delete users with a specific role.
   *
   * @param $role
   *   The role the users need to have to delete them.
   * @usage drush test-users:delete-with-role test_role
   *   Delete all users that have the role test_role.
   * @validate-module-enabled user
   *
   * @command test-users:delete-with-role
   * @aliases duwr,deleteuserswithrole
   */
  public function deleteUsersWithRole($role) {
    // The user module does not offer a way to get all users for a role itself,
    // so we use an entity query.
    $query = \Drupal::entityQuery('user')
      ->condition("roles.target_id", $role);

    $result = $query->execute();

    /** @var \Drupal\Core\Session\AccountInterface[] $accounts */
    $accounts = User::loadMultiple($result);

    if (empty($accounts)) {
      $this->logger()->notice('No users found.');
    }
    else {
      $userList = implode(PHP_EOL, array_map(function ($account) {
        return $account->getDisplayName();
      }, $accounts));
      $msg = 'This will delete these users:' . PHP_EOL . $userList . PHP_EOL . 'Are you sure?';
      $sure = $this->io()->confirm($msg);
      if ($sure) {
        foreach ($accounts as $name => $account) {
          try {
            $account->delete();
            $this->logger()->notice('Deleted user ' . $name);
          }
          catch (EntityStorageException $e) {
            $this->logger()->error('Failed to delete user ' . $name . '. ' . $e->getMessage());
          }
        }
      }
    }
  }

}
